# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the cisco.ios_route_maps module
"""


class Route_mapsArgs(object):  # pylint: disable=R0903
    """The arg spec for the cisco.ios_route_maps module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "config": {
            "type": "list",
            "elements": "dict",
            "options": {
                "route_map": {"type": "str"},
                "entries": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "sequence": {"type": "int"},
                        "action": {"type": "str", "choices": ["deny", "permit"]},
                        "continue_entry": {
                            "type": "dict",
                            "options": {"set": {"type": "bool"}, "entry_sequence": {"type": "int"}},
                        },
                        "description": {"type": "str"},
                        "match": {
                            "type": "dict",
                            "options": {
                                "additional_paths": {
                                    "type": "dict",
                                    "options": {
                                        "all": {"type": "bool"},
                                        "best": {"type": "int"},
                                        "best_range": {
                                            "type": "dict",
                                            "options": {
                                                "lower_limit": {"type": "int"},
                                                "upper_limit": {"type": "int"},
                                            },
                                        },
                                        "group_best": {"type": "bool"},
                                    },
                                },
                                "as_path": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "acls": {"type": "list", "elements": "int"},
                                    },
                                },
                                "clns": {
                                    "type": "dict",
                                    "options": {
                                        "address": {"type": "str"},
                                        "next_hop": {"type": "str"},
                                        "route_source": {"type": "str"},
                                    },
                                },
                                "community": {
                                    "type": "dict",
                                    "options": {
                                        "name": {"type": "list", "elements": "str"},
                                        "exact_match": {"type": "bool"},
                                    },
                                },
                                "extcommunity": {"type": "list", "elements": "str"},
                                "interfaces": {"type": "list", "elements": "str"},
                                "ip": {
                                    "type": "dict",
                                    "options": {
                                        "address": {
                                            "type": "dict",
                                            "options": {
                                                "acls": {"type": "list", "elements": "str"},
                                                "prefix_lists": {"type": "list", "elements": "str"},
                                            },
                                        },
                                        "flowspec": {
                                            "type": "dict",
                                            "options": {
                                                "dest_pfx": {"type": "bool"},
                                                "src_pfx": {"type": "bool"},
                                                "acls": {"type": "list", "elements": "str"},
                                                "prefix_lists": {"type": "list", "elements": "str"},
                                            },
                                        },
                                        "next_hop": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "acls": {"type": "list", "elements": "str"},
                                                "prefix_lists": {"type": "list", "elements": "str"},
                                            },
                                        },
                                        "redistribution_source": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "acls": {"type": "list", "elements": "str"},
                                                "prefix_lists": {"type": "list", "elements": "str"},
                                            },
                                        },
                                        "route_source": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "redistribution_source": {"type": "bool"},
                                                "acls": {"type": "list", "elements": "str"},
                                                "prefix_lists": {"type": "list", "elements": "str"},
                                            },
                                        },
                                    },
                                },
                                "ipv6": {
                                    "type": "dict",
                                    "options": {
                                        "address": {
                                            "type": "dict",
                                            "options": {
                                                "acl": {"type": "str"},
                                                "prefix_list": {"type": "str"},
                                            },
                                        },
                                        "flowspec": {
                                            "type": "dict",
                                            "options": {
                                                "dest_pfx": {"type": "bool"},
                                                "src_pfx": {"type": "bool"},
                                                "acl": {"type": "str"},
                                                "prefix_list": {"type": "str"},
                                            },
                                        },
                                        "next_hop": {
                                            "type": "dict",
                                            "options": {
                                                "acl": {"type": "str"},
                                                "prefix_list": {"type": "str"},
                                            },
                                        },
                                        "route_source": {
                                            "type": "dict",
                                            "options": {
                                                "acl": {"type": "str"},
                                                "prefix_list": {"type": "str"},
                                            },
                                        },
                                    },
                                },
                                "length": {
                                    "type": "dict",
                                    "options": {
                                        "minimum": {"type": "int"},
                                        "maximum": {"type": "int"},
                                    },
                                },
                                "local_preference": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "value": {"type": "list", "elements": "str"},
                                    },
                                },
                                "mdt_group": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "acls": {"type": "list", "elements": "str"},
                                    },
                                },
                                "metric": {
                                    "type": "dict",
                                    "options": {
                                        "value": {"type": "int"},
                                        "external": {"type": "bool"},
                                        "deviation": {"type": "bool"},
                                        "deviation_value": {"type": "int"},
                                    },
                                },
                                "mpls_label": {"type": "bool"},
                                "policy_lists": {"type": "list", "elements": "str"},
                                "route_type": {
                                    "type": "dict",
                                    "options": {
                                        "external": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "type_1": {"type": "bool"},
                                                "type_2": {"type": "bool"},
                                            },
                                        },
                                        "internal": {"type": "bool"},
                                        "level_1": {"type": "bool"},
                                        "level_2": {"type": "bool"},
                                        "local": {"type": "bool"},
                                        "nssa_external": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "type_1": {"type": "bool"},
                                                "type_2": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                                "rpki": {
                                    "type": "dict",
                                    "options": {
                                        "invalid": {"type": "bool"},
                                        "not_found": {"type": "bool"},
                                        "valid": {"type": "bool"},
                                    },
                                },
                                "security_group": {
                                    "type": "dict",
                                    "options": {
                                        "source": {"type": "list", "elements": "int"},
                                        "destination": {"type": "list", "elements": "int"},
                                    },
                                },
                                "source_protocol": {
                                    "type": "dict",
                                    "options": {
                                        "bgp": {"type": "str"},
                                        "connected": {"type": "bool"},
                                        "eigrp": {"type": "int"},
                                        "isis": {"type": "bool"},
                                        "lisp": {"type": "bool"},
                                        "mobile": {"type": "bool"},
                                        "ospf": {"type": "int"},
                                        "ospfv3": {"type": "int"},
                                        "rip": {"type": "bool"},
                                        "static": {"type": "bool"},
                                    },
                                },
                                "tag": {
                                    "type": "dict",
                                    "options": {
                                        "value": {"type": "list", "elements": "str"},
                                        "tag_list": {"type": "list", "elements": "str"},
                                    },
                                },
                                "track": {"type": "int"},
                            },
                        },
                        "set": {
                            "type": "dict",
                            "options": {
                                "aigp_metric": {
                                    "type": "dict",
                                    "options": {
                                        "value": {"type": "int"},
                                        "igp_metric": {"type": "bool"},
                                    },
                                },
                                "as_path": {
                                    "type": "dict",
                                    "options": {
                                        "prepend": {
                                            "type": "dict",
                                            "options": {
                                                "as_number": {"type": "list", "elements": "str"},
                                                "last_as": {"type": "int"},
                                            },
                                        },
                                        "tag": {"type": "bool"},
                                    },
                                },
                                "automatic_tag": {"type": "bool"},
                                "clns": {"type": "str"},
                                "comm_list": {"type": "str"},
                                "community": {
                                    "type": "dict",
                                    "options": {
                                        "number": {"type": "str"},
                                        "additive": {"type": "bool"},
                                        "gshut": {"type": "bool"},
                                        "internet": {"type": "bool"},
                                        "local_as": {"type": "bool"},
                                        "no_advertise": {"type": "bool"},
                                        "no_export": {"type": "bool"},
                                        "none": {"type": "bool"},
                                    },
                                },
                                "dampening": {
                                    "type": "dict",
                                    "options": {
                                        "penalty_half_time": {"type": "int"},
                                        "reuse_route_val": {"type": "int"},
                                        "suppress_route_val": {"type": "int"},
                                        "max_suppress": {"type": "int"},
                                    },
                                },
                                "default": {"type": "str"},
                                "extcomm_list": {"type": "str"},
                                "extcommunity": {
                                    "type": "dict",
                                    "options": {
                                        "cost": {
                                            "type": "dict",
                                            "options": {
                                                "id": {"type": "str"},
                                                "cost_value": {"type": "int"},
                                                "igp": {"type": "bool"},
                                                "pre_bestpath": {"type": "bool"},
                                            },
                                        },
                                        "rt": {
                                            "type": "dict",
                                            "options": {
                                                "address": {"type": "str"},
                                                "range": {
                                                    "type": "dict",
                                                    "options": {
                                                        "lower_limit": {"type": "str"},
                                                        "upper_limit": {"type": "str"},
                                                    },
                                                },
                                                "additive": {"type": "bool"},
                                            },
                                        },
                                        "soo": {"type": "str"},
                                        "vpn_distinguisher": {
                                            "type": "dict",
                                            "options": {
                                                "address": {"type": "str"},
                                                "range": {
                                                    "type": "dict",
                                                    "options": {
                                                        "lower_limit": {"type": "str"},
                                                        "upper_limit": {"type": "str"},
                                                    },
                                                },
                                                "additive": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                                "global_route": {"type": "bool"},
                                "interfaces": {"type": "list", "elements": "str"},
                                "ip": {
                                    "type": "dict",
                                    "options": {
                                        "address": {"type": "str"},
                                        "df": {"choices": [0, 1], "type": "int"},
                                        "global_route": {
                                            "type": "dict",
                                            "options": {
                                                "address": {"type": "str"},
                                                "verify_availability": {
                                                    "type": "dict",
                                                    "options": {
                                                        "address": {"type": "str"},
                                                        "sequence": {"type": "int"},
                                                        "track": {"type": "int"},
                                                    },
                                                },
                                            },
                                        },
                                        "next_hop": {
                                            "type": "dict",
                                            "options": {
                                                "address": {"type": "str"},
                                                "dynamic": {"type": "bool"},
                                                "encapsulate": {"type": "str"},
                                                "peer_address": {"type": "bool"},
                                                "recursive": {
                                                    "type": "dict",
                                                    "options": {
                                                        "address": {"type": "str"},
                                                        "global_route": {"type": "bool"},
                                                        "vrf": {"type": "str"},
                                                    },
                                                },
                                                "self": {"type": "bool"},
                                                "verify_availability": {
                                                    "type": "dict",
                                                    "options": {
                                                        "set": {"type": "bool"},
                                                        "address": {},
                                                        "sequence": {"type": "int"},
                                                        "track": {"type": "int"},
                                                    },
                                                },
                                            },
                                        },
                                        "precedence": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "critical": {"type": "bool"},
                                                "flash": {"type": "bool"},
                                                "flash_override": {"type": "bool"},
                                                "immediate": {"type": "bool"},
                                                "internet": {"type": "bool"},
                                                "network": {"type": "bool"},
                                                "priority": {"type": "bool"},
                                                "routine": {"type": "bool"},
                                            },
                                        },
                                        "qos_group": {"type": "int"},
                                        "tos": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "max_reliability": {"type": "bool"},
                                                "max_throughput": {"type": "bool"},
                                                "min_delay": {"type": "bool"},
                                                "min_monetary_cost": {"type": "bool"},
                                                "normal": {"type": "bool"},
                                            },
                                        },
                                        "vrf": {
                                            "type": "dict",
                                            "options": {
                                                "name": {"type": "str"},
                                                "address": {"type": "str"},
                                                "verify_availability": {
                                                    "type": "dict",
                                                    "options": {
                                                        "set": {"type": "bool"},
                                                        "address": {},
                                                        "sequence": {"type": "int"},
                                                        "track": {"type": "int"},
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "ipv6": {
                                    "type": "dict",
                                    "options": {
                                        "address": {"type": "str"},
                                        "default": {"type": "bool"},
                                        "global_route": {
                                            "type": "dict",
                                            "options": {
                                                "address": {"type": "str"},
                                                "verify_availability": {
                                                    "type": "dict",
                                                    "options": {
                                                        "address": {"type": "str"},
                                                        "sequence": {"type": "int"},
                                                        "track": {"type": "int"},
                                                    },
                                                },
                                            },
                                        },
                                        "next_hop": {
                                            "type": "dict",
                                            "options": {
                                                "address": {"type": "str"},
                                                "encapsulate": {"type": "str"},
                                                "peer_address": {"type": "bool"},
                                                "recursive": {"type": "str"},
                                            },
                                        },
                                        "precedence": {"type": "int"},
                                        "vrf": {
                                            "type": "dict",
                                            "options": {
                                                "name": {"type": "str"},
                                                "verify_availability": {
                                                    "type": "dict",
                                                    "options": {
                                                        "address": {},
                                                        "sequence": {"type": "int"},
                                                        "track": {"type": "int"},
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "level": {
                                    "type": "dict",
                                    "options": {
                                        "level_1": {"type": "bool"},
                                        "level_1_2": {"type": "bool"},
                                        "level_2": {"type": "bool"},
                                        "nssa_only": {"type": "bool"},
                                    },
                                },
                                "lisp": {"type": "str"},
                                "local_preference": {"type": "int"},
                                "metric": {
                                    "type": "dict",
                                    "options": {
                                        "deviation": {"choices": ["plus", "minus"], "type": "str"},
                                        "metric_value": {"type": "int"},
                                        "eigrp_delay": {"type": "int"},
                                        "metric_reliability": {"type": "int"},
                                        "metric_bandwidth": {"type": "int"},
                                        "mtu": {"type": "int"},
                                    },
                                },
                                "metric_type": {
                                    "type": "dict",
                                    "options": {
                                        "external": {"type": "bool"},
                                        "internal": {"type": "bool"},
                                        "type_1": {"type": "bool"},
                                        "type_2": {"type": "bool"},
                                    },
                                },
                                "mpls_label": {"type": "bool"},
                                "origin": {
                                    "type": "dict",
                                    "options": {
                                        "igp": {"type": "bool"},
                                        "incomplete": {"type": "bool"},
                                    },
                                },
                                "tag": {"type": "str"},
                                "traffic_index": {"type": "int"},
                                "vrf": {"type": "str"},
                                "weight": {"type": "int"},
                            },
                        },
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "parsed",
                "rendered",
            ],
            "default": "merged",
        },
    }



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: mq_broker_info
version_added: 6.0.0
short_description: Retrieve MQ Broker details
description:
  - Get details about a broker.
author:
  - FCO (@fotto)
options:
  broker_id:
    description: Get details for broker with specified ID.
    type: str
  broker_name:
    description:
      - Get details for broker with specified Name.
      - Is ignored if I(broker_id) is specified.
    type: str
extends_documentation_fragment:
  - amazon.aws.boto3
  - amazon.aws.common.modules
  - amazon.aws.region.modules
"""


EXAMPLES = r"""
- name: get current broker settings by id
  community.aws.mq_broker_info:
    broker_id: "aws-mq-broker-id"
  register: broker_info

- name: get current broker settings by name setting all credential parameters explicitly
  community.aws.mq_broker_info:
    broker_name: "aws-mq-broker-name"
  register: broker_info
"""

RETURN = r"""
broker:
    description: API response of describe_broker() converted to snake yaml.
    type: dict
    returned: success
"""

try:
    import botocore
except ImportError:
    # handled by AnsibleAWSModule
    pass

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule


def get_broker_id(conn, module):
    try:
        broker_name = module.params["broker_name"]
        broker_id = None
        response = conn.list_brokers(MaxResults=100)
        for broker in response["BrokerSummaries"]:
            if broker["BrokerName"] == broker_name:
                broker_id = broker["BrokerId"]
                break
        return broker_id
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't list broker brokers.")


def get_broker_info(conn, module, broker_id):
    try:
        return conn.describe_broker(BrokerId=broker_id)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        if module.check_mode:
            module.exit_json(broker={"broker_id": broker_id, "broker_name": "fakeName"})
        else:
            module.fail_json_aws(e, msg="Couldn't get broker details.")


def main():
    argument_spec = dict(broker_id=dict(type="str"), broker_name=dict(type="str"))
    required_one_of = (
        (
            "broker_name",
            "broker_id",
        ),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_one_of=required_one_of,
        supports_check_mode=True,
    )
    broker_id = module.params["broker_id"]
    broker_name = module.params["broker_name"]

    connection = module.client("mq")

    try:
        if not broker_id:
            broker_id = get_broker_id(connection, module)
        if not broker_id:
            if module.check_mode:
                module.exit_json(
                    broker={"broker_id": "fakeId", "broker_name": broker_name if broker_name else "fakeName"}
                )
        result = get_broker_info(connection, module, broker_id)
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)
    #
    module.exit_json(broker=camel_dict_to_snake_dict(result, ignore_list=["Tags"]))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2023, Raman Babich <ramanbabich@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: digital_ocean_project_resource_info
short_description: Gather information about DigitalOcean Project Resources
description:
    - This module can be used to gather information about Project Resources.
author: "Raman Babich (@raman-babich)"
version_added: 1.25.0

options:
  id:
    description:
      - Project ID that can be used to identify and reference a project.
      - If C(id) and C(name) are not specified default project will be used.
    type: str
  name:
    description:
      - Project name that can be used to identify and reference a project.
      - If C(id) and C(name) are not specified default project will be used.
    type: str

extends_documentation_fragment:
- community.digitalocean.digital_ocean
"""

EXAMPLES = r"""
- name: Get project resources by id
  community.digitalocean.digital_ocean_project_resource_info:
    id: cb1ef55e-3cd8-4c7c-aa5d-07c32bf41627

- name: Get project resources by name
  community.digitalocean.digital_ocean_project_resource_info:
    name: my-project-name

- name: Get default project resources
  community.digitalocean.digital_ocean_project_resource_info:
"""

RETURN = r"""
data:
  description: "DigitalOcean project resources information"
  elements: dict
  returned: success
  type: list
  sample:
    - urn: "do:droplet:13457723"
      assigned_at: "2018-09-28T19:26:37Z"
      links:
        self: "https://api.digitalocean.com/v2/droplets/13457723"
      status: "ok"
    - urn: "do:domain:example.com"
      assigned_at: "2019-03-31T16:24:14Z"
      links:
        self: "https://api.digitalocean.com/v2/domains/example.com"
      status: "ok"
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
    DigitalOceanProjects,
)


def run(module):
    rest = DigitalOceanHelper(module)
    projects = DigitalOceanProjects(module, rest)
    if module.params["id"]:
        err_msg, resources = projects.get_resources_by_id(module.params["id"])
    elif module.params["name"]:
        err_msg, resources = projects.get_resources_by_name(module.params["name"])
    else:
        err_msg, resources = projects.get_resources_of_default()

    if err_msg:
        module.fail_json(msg=err_msg)
    module.exit_json(data=resources["resources"])


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        name=dict(type="str"),
        id=dict(type="str"),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[("id", "name")],
        supports_check_mode=True,
    )
    run(module)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2022 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):

    DOCUMENTATION = r'''
options:
    icann_only:
        description:
            - This controls whether only entries from the ICANN section of the Public Suffix List are used,
              or also entries from the Private section. For example, C(.co.uk) is in the ICANN section,
              but C(github.io) is in the Private section.
        type: boolean
        default: false
'''

    PUBLIC_SUFFIX = r'''
options:
    keep_unknown_suffix:
        description:
            - This treats unknown TLDs as valid public suffixes. So for example the public suffix
              of C(example.tlddoesnotexist) is C(.tlddoesnotexist) if this is V(true). If set to
              V(false), it will return an empty string in this case.
            - This option corresponds to whether the global wildcard rule C(*) in the Public
              Suffix List is used or not.
        type: boolean
        default: true
'''

    REGISTERABLE_DOMAIN = r'''
options:
    only_if_registerable:
        description:
            - This controls the behavior in case there is no label in front of the public suffix.
              This is the case if the DNS name itself is a public suffix.
            - If set to V(false), in this case the public suffix is treated as a registrable domain.
            - If set to V(true) (default), the registrable domain of a public suffix is interpreted as an
              empty string.
        type: boolean
        default: true
    keep_unknown_suffix:
        description:
            - This treats unknown TLDs as valid public suffixes. So for example the public suffix of
              C(example.tlddoesnotexist) is C(.tlddoesnotexist) if this is V(true), and hence the
              registrable domain of C(www.example.tlddoesnotexist) is C(example.tlddoesnotexist).
              If set to V(false), the registrable domain of C(www.example.tlddoesnotexist) is
              C(tlddoesnotexist).
            - This option corresponds to whether the global wildcard rule C(*) in the Public Suffix List
              is used or not.
        type: boolean
        default: true
'''

    GET = r'''
options:
    normalize_result:
        description:
            - This controls whether the result is reconstructed from the normalized name used during lookup.
              During normalization, ulabels are converted to alabels, and every label is converted to lowercase.
              For example, the ulabel C(ëçãmplê) is converted to C(xn--mpl-llatwb) (puny-code), and
              C(Example.COM) is converted to C(example.com).
        type: boolean
        default: false
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: subscriber_imsi_info
short_description: Information module for Subscriber Imsi Info
description:
- Get Subscriber Imsi Info by id.
version_added: '2.8.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  imsi:
    description:
    - Imsi path parameter. IMSI parameter.
    type: str
requirements:
- ciscoisesdk >= 2.0.1
- python >= 3.5
notes:
  - SDK Method used are
    subscriber.Subscriber.get_subscriber_by_i_m_s_i,

  - Paths used are
    get /api/v1/fiveg/subscriber/imsi/{imsi},

"""

EXAMPLES = r"""
- name: Get Subscriber Imsi Info by id
  cisco.ise.subscriber_imsi_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    imsi: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "createTime": "string",
      "enabled": true,
      "friendlyName": "string",
      "id": "string",
      "identityGroups": "string",
      "imeis": "string",
      "imsi": "string",
      "ki": "string",
      "link": {
        "href": "string",
        "rel": "string",
        "type": "string"
      },
      "opc": "string",
      "updateTime": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Brian Scholer (@briantist)
# Copyright: (c) 2015, Jon Hawkesworth (@jhawkesworth) <figs@unity.demon.co.uk>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_environment
short_description: Modify environment variables on windows hosts
description:
- Uses .net Environment to set or remove environment variables and can set at User, Machine or Process level.
- User level environment variables will be set, but not available until the user has logged off and on again.
options:
  state:
    description:
    - Set to C(present) to ensure environment variable is set.
    - Set to C(absent) to ensure it is removed.
    - When using I(variables), do not set this option.
    type: str
    choices: [ absent, present ]
  name:
    description:
    - The name of the environment variable. Required when I(state=absent).
    type: str
  value:
    description:
    - The value to store in the environment variable.
    - Must be set when I(state=present) and cannot be an empty string.
    - Should be omitted for I(state=absent) and I(variables).
    type: str
  variables:
    description:
    - A dictionary where multiple environment variables can be defined at once.
    - Not valid when I(state) is set. Variables with a value will be set (C(present)) and variables with an empty value will be unset (C(absent)).
    - I(level) applies to all vars defined this way.
    type: dict
    version_added: '1.3.0'
  level:
    description:
    - The level at which to set the environment variable.
    - Use C(machine) to set for all users.
    - Use C(user) to set for the current user that ansible is connected as.
    - Use C(process) to set for the current process.  Probably not that useful.
    type: str
    required: yes
    choices: [ machine, process, user ]
notes:
- This module is best-suited for setting the entire value of an
  environment variable. For safe element-based management of
  path-like environment vars, use the M(ansible.windows.win_path) module.
- This module does not broadcast change events.
  This means that the minority of windows applications which can have
  their environment changed without restarting will not be notified and
  therefore will need restarting to pick up new environment settings.
  User level environment variables will require the user to log out
  and in again before they become available.
- In the return, C(before_value) and C(value) will be set to the last values
  when using I(variables). It's best to use C(values) in that case if you need
  to find a specific variable's before and after values.
seealso:
- module: ansible.windows.win_path
author:
- Jon Hawkesworth (@jhawkesworth)
- Brian Scholer (@briantist)
'''

EXAMPLES = r'''
- name: Set an environment variable for all users
  ansible.windows.win_environment:
    state: present
    name: TestVariable
    value: Test value
    level: machine

- name: Remove an environment variable for the current user
  ansible.windows.win_environment:
    state: absent
    name: TestVariable
    level: user

- name: Set several variables at once
  ansible.windows.win_environment:
    level: machine
    variables:
      TestVariable: Test value
      CUSTOM_APP_VAR: 'Very important value'
      ANOTHER_VAR: '{{ my_ansible_var }}'

- name: Set and remove multiple variables at once
  ansible.windows.win_environment:
    level: user
    variables:
      TestVariable: Test value
      CUSTOM_APP_VAR: 'Very important value'
      ANOTHER_VAR: '{{ my_ansible_var }}'
      UNWANTED_VAR: ''  # < this will be removed
'''

RETURN = r'''
before_value:
  description: the value of the environment key before a change, this is null if it didn't exist
  returned: always
  type: str
  sample: C:\Windows\System32
value:
  description: the value the environment key has been set to, this is null if removed
  returned: always
  type: str
  sample: C:\Program Files\jdk1.8
values:
  description: "dictionary of before and after values; each key is a variable name, each value is
  another dict with C(before), C(after), and C(changed) keys"
  returned: always
  type: dict
  version_added: '1.3.0'
'''



# -*- coding: utf-8 -*-
#
# Copyright (c) 2024, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.crypto.plugins.module_utils.crypto.math import (
    convert_int_to_hex,
)


def th(number):
    abs_number = abs(number)
    mod_10 = abs_number % 10
    mod_100 = abs_number % 100
    if mod_100 not in (11, 12, 13):
        if mod_10 == 1:
            return 'st'
        if mod_10 == 2:
            return 'nd'
        if mod_10 == 3:
            return 'rd'
    return 'th'


def parse_serial(value):
    """
    Given a colon-separated string of hexadecimal byte values, converts it to an integer.
    """
    value = to_native(value)
    result = 0
    for i, part in enumerate(value.split(':')):
        try:
            part_value = int(part, 16)
            if part_value < 0 or part_value > 255:
                raise ValueError('the value is not in range [0, 255]')
        except ValueError as exc:
            raise ValueError("The {idx}{th} part {part!r} is not a hexadecimal number in range [0, 255]: {exc}".format(
                idx=i + 1, th=th(i + 1), part=part, exc=exc))
        result = (result << 8) | part_value
    return result


def to_serial(value):
    """
    Given an integer, converts its absolute value to a colon-separated string of hexadecimal byte values.
    """
    value = convert_int_to_hex(value).upper()
    if len(value) % 2 != 0:
        value = '0' + value
    return ':'.join(value[i:i + 2] for i in range(0, len(value), 2))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


import abc
import datetime
import errno
import hashlib
import os
import re

from ansible.module_utils import six
from ansible.module_utils.common.text.converters import to_native, to_bytes

from ansible_collections.community.crypto.plugins.module_utils.crypto.pem import (
    identify_pem_format,
)

try:
    from OpenSSL import crypto
    HAS_PYOPENSSL = True
except (ImportError, AttributeError):
    # Error handled in the calling module.
    HAS_PYOPENSSL = False

try:
    from cryptography import x509
    from cryptography.hazmat.backends import default_backend as cryptography_backend
    from cryptography.hazmat.primitives.serialization import load_pem_private_key
    from cryptography.hazmat.primitives import hashes
    from cryptography.hazmat.primitives import serialization
except ImportError:
    # Error handled in the calling module.
    pass

from .basic import (
    OpenSSLObjectError,
    OpenSSLBadPassphraseError,
)


# This list of preferred fingerprints is used when prefer_one=True is supplied to the
# fingerprinting methods.
PREFERRED_FINGERPRINTS = (
    'sha256', 'sha3_256', 'sha512', 'sha3_512', 'sha384', 'sha3_384', 'sha1', 'md5'
)


def get_fingerprint_of_bytes(source, prefer_one=False):
    """Generate the fingerprint of the given bytes."""

    fingerprint = {}

    try:
        algorithms = hashlib.algorithms
    except AttributeError:
        try:
            algorithms = hashlib.algorithms_guaranteed
        except AttributeError:
            return None

    if prefer_one:
        # Sort algorithms to have the ones in PREFERRED_FINGERPRINTS at the beginning
        prefered_algorithms = [algorithm for algorithm in PREFERRED_FINGERPRINTS if algorithm in algorithms]
        prefered_algorithms += sorted([algorithm for algorithm in algorithms if algorithm not in PREFERRED_FINGERPRINTS])
        algorithms = prefered_algorithms

    for algo in algorithms:
        f = getattr(hashlib, algo)
        try:
            h = f(source)
        except ValueError:
            # This can happen for hash algorithms not supported in FIPS mode
            # (https://github.com/ansible/ansible/issues/67213)
            continue
        try:
            # Certain hash functions have a hexdigest() which expects a length parameter
            pubkey_digest = h.hexdigest()
        except TypeError:
            pubkey_digest = h.hexdigest(32)
        fingerprint[algo] = ':'.join(pubkey_digest[i:i + 2] for i in range(0, len(pubkey_digest), 2))
        if prefer_one:
            break

    return fingerprint


def get_fingerprint_of_privatekey(privatekey, backend='cryptography', prefer_one=False):
    """Generate the fingerprint of the public key. """

    if backend == 'cryptography':
        publickey = privatekey.public_key().public_bytes(
            serialization.Encoding.DER,
            serialization.PublicFormat.SubjectPublicKeyInfo
        )

    return get_fingerprint_of_bytes(publickey, prefer_one=prefer_one)


def get_fingerprint(path, passphrase=None, content=None, backend='cryptography', prefer_one=False):
    """Generate the fingerprint of the public key. """

    privatekey = load_privatekey(path, passphrase=passphrase, content=content, check_passphrase=False, backend=backend)

    return get_fingerprint_of_privatekey(privatekey, backend=backend, prefer_one=prefer_one)


def load_privatekey(path, passphrase=None, check_passphrase=True, content=None, backend='cryptography'):
    """Load the specified OpenSSL private key.

    The content can also be specified via content; in that case,
    this function will not load the key from disk.
    """

    try:
        if content is None:
            with open(path, 'rb') as b_priv_key_fh:
                priv_key_detail = b_priv_key_fh.read()
        else:
            priv_key_detail = content
    except (IOError, OSError) as exc:
        raise OpenSSLObjectError(exc)

    if backend == 'pyopenssl':

        # First try: try to load with real passphrase (resp. empty string)
        # Will work if this is the correct passphrase, or the key is not
        # password-protected.
        try:
            result = crypto.load_privatekey(crypto.FILETYPE_PEM,
                                            priv_key_detail,
                                            to_bytes(passphrase or ''))
        except crypto.Error as e:
            if len(e.args) > 0 and len(e.args[0]) > 0:
                if e.args[0][0][2] in ('bad decrypt', 'bad password read'):
                    # This happens in case we have the wrong passphrase.
                    if passphrase is not None:
                        raise OpenSSLBadPassphraseError('Wrong passphrase provided for private key!')
                    else:
                        raise OpenSSLBadPassphraseError('No passphrase provided, but private key is password-protected!')
            raise OpenSSLObjectError('Error while deserializing key: {0}'.format(e))
        if check_passphrase:
            # Next we want to make sure that the key is actually protected by
            # a passphrase (in case we did try the empty string before, make
            # sure that the key is not protected by the empty string)
            try:
                crypto.load_privatekey(crypto.FILETYPE_PEM,
                                       priv_key_detail,
                                       to_bytes('y' if passphrase == 'x' else 'x'))
                if passphrase is not None:
                    # Since we can load the key without an exception, the
                    # key is not password-protected
                    raise OpenSSLBadPassphraseError('Passphrase provided, but private key is not password-protected!')
            except crypto.Error as e:
                if passphrase is None and len(e.args) > 0 and len(e.args[0]) > 0:
                    if e.args[0][0][2] in ('bad decrypt', 'bad password read'):
                        # The key is obviously protected by the empty string.
                        # Do not do this at home (if it's possible at all)...
                        raise OpenSSLBadPassphraseError('No passphrase provided, but private key is password-protected!')
    elif backend == 'cryptography':
        try:
            result = load_pem_private_key(priv_key_detail,
                                          None if passphrase is None else to_bytes(passphrase),
                                          cryptography_backend())
        except TypeError:
            raise OpenSSLBadPassphraseError('Wrong or empty passphrase provided for private key')
        except ValueError:
            raise OpenSSLBadPassphraseError('Wrong passphrase provided for private key')

    return result


def load_publickey(path=None, content=None, backend=None):
    if content is None:
        if path is None:
            raise OpenSSLObjectError('Must provide either path or content')
        try:
            with open(path, 'rb') as b_priv_key_fh:
                content = b_priv_key_fh.read()
        except (IOError, OSError) as exc:
            raise OpenSSLObjectError(exc)

    if backend == 'cryptography':
        try:
            return serialization.load_pem_public_key(content, backend=cryptography_backend())
        except Exception as e:
            raise OpenSSLObjectError('Error while deserializing key: {0}'.format(e))


def load_certificate(path, content=None, backend='cryptography', der_support_enabled=False):
    """Load the specified certificate."""

    try:
        if content is None:
            with open(path, 'rb') as cert_fh:
                cert_content = cert_fh.read()
        else:
            cert_content = content
    except (IOError, OSError) as exc:
        raise OpenSSLObjectError(exc)
    if backend == 'pyopenssl':
        if der_support_enabled is False or identify_pem_format(cert_content):
            return crypto.load_certificate(crypto.FILETYPE_PEM, cert_content)
        elif der_support_enabled:
            raise OpenSSLObjectError('Certificate in DER format is not supported by the pyopenssl backend.')
    elif backend == 'cryptography':
        if der_support_enabled is False or identify_pem_format(cert_content):
            try:
                return x509.load_pem_x509_certificate(cert_content, cryptography_backend())
            except ValueError as exc:
                raise OpenSSLObjectError(exc)
        elif der_support_enabled:
            try:
                return x509.load_der_x509_certificate(cert_content, cryptography_backend())
            except ValueError as exc:
                raise OpenSSLObjectError('Cannot parse DER certificate: {0}'.format(exc))


def load_certificate_request(path, content=None, backend='cryptography'):
    """Load the specified certificate signing request."""
    try:
        if content is None:
            with open(path, 'rb') as csr_fh:
                csr_content = csr_fh.read()
        else:
            csr_content = content
    except (IOError, OSError) as exc:
        raise OpenSSLObjectError(exc)
    if backend == 'cryptography':
        try:
            return x509.load_pem_x509_csr(csr_content, cryptography_backend())
        except ValueError as exc:
            raise OpenSSLObjectError(exc)


def parse_name_field(input_dict, name_field_name=None):
    """Take a dict with key: value or key: list_of_values mappings and return a list of tuples"""
    error_str = '{key}' if name_field_name is None else '{key} in {name}'

    result = []
    for key, value in input_dict.items():
        if isinstance(value, list):
            for entry in value:
                if not isinstance(entry, six.string_types):
                    raise TypeError(('Values %s must be strings' % error_str).format(key=key, name=name_field_name))
                if not entry:
                    raise ValueError(('Values for %s must not be empty strings' % error_str).format(key=key))
                result.append((key, entry))
        elif isinstance(value, six.string_types):
            if not value:
                raise ValueError(('Value for %s must not be an empty string' % error_str).format(key=key))
            result.append((key, value))
        else:
            raise TypeError(('Value for %s must be either a string or a list of strings' % error_str).format(key=key))
    return result


def parse_ordered_name_field(input_list, name_field_name):
    """Take a dict with key: value or key: list_of_values mappings and return a list of tuples"""

    result = []
    for index, entry in enumerate(input_list):
        if len(entry) != 1:
            raise ValueError(
                'Entry #{index} in {name} must be a dictionary with exactly one key-value pair'.format(
                    name=name_field_name, index=index + 1))
        try:
            result.extend(parse_name_field(entry, name_field_name=name_field_name))
        except (TypeError, ValueError) as exc:
            raise ValueError(
                'Error while processing entry #{index} in {name}: {error}'.format(
                    name=name_field_name, index=index + 1, error=exc))
    return result


def convert_relative_to_datetime(relative_time_string):
    """Get a datetime.datetime or None from a string in the time format described in sshd_config(5)"""

    parsed_result = re.match(
        r"^(?P<prefix>[+-])((?P<weeks>\d+)[wW])?((?P<days>\d+)[dD])?((?P<hours>\d+)[hH])?((?P<minutes>\d+)[mM])?((?P<seconds>\d+)[sS]?)?$",
        relative_time_string)

    if parsed_result is None or len(relative_time_string) == 1:
        # not matched or only a single "+" or "-"
        return None

    offset = datetime.timedelta(0)
    if parsed_result.group("weeks") is not None:
        offset += datetime.timedelta(weeks=int(parsed_result.group("weeks")))
    if parsed_result.group("days") is not None:
        offset += datetime.timedelta(days=int(parsed_result.group("days")))
    if parsed_result.group("hours") is not None:
        offset += datetime.timedelta(hours=int(parsed_result.group("hours")))
    if parsed_result.group("minutes") is not None:
        offset += datetime.timedelta(
            minutes=int(parsed_result.group("minutes")))
    if parsed_result.group("seconds") is not None:
        offset += datetime.timedelta(
            seconds=int(parsed_result.group("seconds")))

    if parsed_result.group("prefix") == "+":
        return datetime.datetime.utcnow() + offset
    else:
        return datetime.datetime.utcnow() - offset


def get_relative_time_option(input_string, input_name, backend='cryptography'):
    """Return an absolute timespec if a relative timespec or an ASN1 formatted
       string is provided.

       The return value will be a datetime object for the cryptography backend,
       and a ASN1 formatted string for the pyopenssl backend."""
    result = to_native(input_string)
    if result is None:
        raise OpenSSLObjectError(
            'The timespec "%s" for %s is not valid' %
            input_string, input_name)
    # Relative time
    if result.startswith("+") or result.startswith("-"):
        result_datetime = convert_relative_to_datetime(result)
        if backend == 'pyopenssl':
            return result_datetime.strftime("%Y%m%d%H%M%SZ")
        elif backend == 'cryptography':
            return result_datetime
    # Absolute time
    if backend == 'cryptography':
        for date_fmt in ['%Y%m%d%H%M%SZ', '%Y%m%d%H%MZ', '%Y%m%d%H%M%S%z', '%Y%m%d%H%M%z']:
            try:
                return datetime.datetime.strptime(result, date_fmt)
            except ValueError:
                pass

        raise OpenSSLObjectError(
            'The time spec "%s" for %s is invalid' %
            (input_string, input_name)
        )


def select_message_digest(digest_string):
    digest = None
    if digest_string == 'sha256':
        digest = hashes.SHA256()
    elif digest_string == 'sha384':
        digest = hashes.SHA384()
    elif digest_string == 'sha512':
        digest = hashes.SHA512()
    elif digest_string == 'sha1':
        digest = hashes.SHA1()
    elif digest_string == 'md5':
        digest = hashes.MD5()
    return digest


@six.add_metaclass(abc.ABCMeta)
class OpenSSLObject(object):

    def __init__(self, path, state, force, check_mode):
        self.path = path
        self.state = state
        self.force = force
        self.name = os.path.basename(path)
        self.changed = False
        self.check_mode = check_mode

    def check(self, module, perms_required=True):
        """Ensure the resource is in its desired state."""

        def _check_state():
            return os.path.exists(self.path)

        def _check_perms(module):
            file_args = module.load_file_common_arguments(module.params)
            if module.check_file_absent_if_check_mode(file_args['path']):
                return False
            return not module.set_fs_attributes_if_different(file_args, False)

        if not perms_required:
            return _check_state()

        return _check_state() and _check_perms(module)

    @abc.abstractmethod
    def dump(self):
        """Serialize the object into a dictionary."""

        pass

    @abc.abstractmethod
    def generate(self):
        """Generate the resource."""

        pass

    def remove(self, module):
        """Remove the resource from the filesystem."""
        if self.check_mode:
            if os.path.exists(self.path):
                self.changed = True
            return

        try:
            os.remove(self.path)
            self.changed = True
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise OpenSSLObjectError(exc)
            else:
                pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2014, Trond Hindenes <trond@hindenes.com>, and others
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_package
short_description: Installs/uninstalls an installable package
description:
- Installs or uninstalls software packages for Windows.
- Supports C(.exe), C(.msi), C(.msp), C(.appx), C(.appxbundle), C(.msix),
  and C(.msixbundle).
- These packages can be sourced from the local file system, network file share
  or a url.
- See I(provider) for more info on each package type that is supported.
options:
  arguments:
    description:
    - Any arguments the installer needs to either install or uninstall the
      package.
    - If the package is an MSI do not supply the C(/qn), C(/log) or
      C(/norestart) arguments.
    - This is only used for the C(msi), C(msp), and C(registry) providers.
    - Can be a list of arguments and the module will escape the arguments as
      necessary, it is recommended to use a string when dealing with MSI
      packages due to the unique escaping issues with msiexec.
    - When using a list of arguments each item in the list is considered to be
      a single argument. As such, if an argument in the list contains a space
      then Ansible will quote this to ensure that this is seen by Windows as
      a single argument. Should this behaviour not be what is required, the
      argument should be split into two separate list items. See the examples
      section for more detail.
    type: raw
  chdir:
    description:
    - Set the specified path as the current working directory before installing
      or uninstalling a package.
    - This is only used for the C(msi), C(msp), and C(registry) providers.
    type: path
  creates_path:
    description:
    - Will check the existence of the path specified and use the result to
      determine whether the package is already installed.
    - You can use this in conjunction with C(product_id) and other C(creates_*).
    type: path
  creates_service:
    description:
    - Will check the existing of the service specified and use the result to
      determine whether the package is already installed.
    - You can use this in conjunction with C(product_id) and other C(creates_*).
    type: str
  creates_version:
    description:
    - Will check the file version property of the file at C(creates_path) and
      use the result to determine whether the package is already installed.
    - C(creates_path) MUST be set and is a file.
    - You can use this in conjunction with C(product_id) and other C(creates_*).
    type: str
  expected_return_code:
    description:
    - One or more return codes from the package installation that indicates
      success.
    - The return codes are read as a signed integer, any values greater than
      2147483647 need to be represented as the signed equivalent, i.e.
      C(4294967295) is C(-1).
    - To convert a unsigned number to the signed equivalent you can run
      "[Int32]("0x{0:X}" -f ([UInt32]3221225477))".
    - A return code of C(3010) usually means that a reboot is required, the
      C(reboot_required) return value is set if the return code is C(3010).
    - This is only used for the C(msi), C(msp), and C(registry) providers.
    type: list
    elements: int
    default: [0, 3010]
  log_path:
    description:
    - Specifies the path to a log file that is persisted after a package is
      installed or uninstalled.
    - This is only used for the C(msi) or C(msp) provider.
    - When omitted, a temporary log file is used instead for those providers.
    - This is only valid for MSI files, use C(arguments) for the C(registry)
      provider.
    type: path
  path:
    description:
    - Location of the package to be installed or uninstalled.
    - This package can either be on the local file system, network share or a
      url.
    - When C(state=present), C(product_id) is not set and the path is a URL,
      this file will always be downloaded to a temporary directory for
      idempotency checks, otherwise the file will only be downloaded if the
      package has not been installed based on the C(product_id) checks.
    - If C(state=present) then this value MUST be set.
    - If C(state=absent) then this value does not need to be set if
      C(product_id) is.
    type: str
  product_id:
    description:
    - The product id of the installed packaged.
    - This is used for checking whether the product is already installed and
      getting the uninstall information if C(state=absent).
    - For msi packages, this is the C(ProductCode) (GUID) of the package. This
      can be found under the same registry paths as the C(registry) provider.
    - For msp packages, this is the C(PatchCode) (GUID) of the package which
      can found under the C(Details -> Revision number) of the file's properties.
    - For msix packages, this is the C(Name) or C(PackageFullName) of the
      package found under the C(Get-AppxPackage) cmdlet.
    - For registry (exe) packages, this is the registry key name under the
      registry paths specified in I(provider).
    - This value is ignored if C(path) is set to a local accesible file path
      and the package is not an C(exe).
    - This SHOULD be set when the package is an C(exe), or the path is a url
      or a network share and credential delegation is not being used. The
      C(creates_*) options can be used instead but is not recommended.
    type: str
  provider:
    description:
    - Set the package provider to use when searching for a package.
    - The C(auto) provider will select the proper provider if I(path)
      otherwise it scans all the other providers based on the I(product_id).
    - The C(msi) provider scans for MSI packages installed on a machine wide
      and current user context based on the C(ProductCode) of the MSI.
    - The C(msix) provider is used to install C(.appx), C(.msix),
      C(.appxbundle), or C(.msixbundle) packages. These packages are only
      installed or removed on the current use. The host must be set to allow
      sideloaded apps or in developer mode. See the examples for how to enable
      this. If a package is already installed but C(path) points to an updated
      package, this will be installed over the top of the existing one.
    - The C(msp) provider scans for all MSP patches installed on a machine wide
      and current user context based on the C(PatchCode) of the MSP. A C(msp)
      will be applied or removed on all C(msi) products that it applies to and
      is installed. If the patch is obsoleted or superseded then no action will
      be taken.
    - The C(registry) provider is used for traditional C(exe) installers and
      uses the following registry path to determine if a product was installed;
      C(HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall),
      C(HKLM:\Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall),
      C(HKCU:\Software\Microsoft\Windows\CurrentVersion\Uninstall), and
      C(HKCU:\Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall).
    choices:
    - auto
    - msi
    - msix
    - msp
    - registry
    default: auto
    type: str
  state:
    description:
    - Whether to install or uninstall the package.
    - The module uses I(product_id) to determine whether the package is
      installed or not.
    - For all providers but C(auto), the I(path) can be used for idempotency
      checks if it is locally accesible filesystem path.
    type: str
    choices: [ absent, present ]
    default: present
  wait_for_children:
    description:
    - The module will wait for the process it spawns to finish but any
      processes spawned in that child process as ignored.
    - Set to C(true) to wait for all descendent processes to finish before the
      module returns.
    - This is useful if the install/uninstaller is just a wrapper which then
      calls the actual installer as its own child process. When this option is
      C(true) then the module will wait for both processes to finish before
      returning.
    - This should not be required for most installers and setting to C(true)
      could result in the module not returning until the process it is waiting
      for has been stopped manually.
    - Requires Windows Server 2012 or Windows 8 or newer to use.
    type: bool
    default: no
    version_added: 1.3.0
extends_documentation_fragment:
- ansible.windows.web_request

notes:
- When C(state=absent) and the product is an exe, the path may be different
  from what was used to install the package originally. If path is not set then
  the path used will be what is set under C(QuietUninstallString) or
  C(UninstallString) in the registry for that I(product_id).
- By default all msi installs and uninstalls will be run with the arguments
  C(/log, /qn, /norestart).
- All the installation checks under C(product_id) and C(creates_*) add
  together, if one fails then the program is considered to be absent.
seealso:
- module: chocolatey.chocolatey.win_chocolatey
- module: community.windows.win_hotfix
- module: ansible.windows.win_updates
author:
- Trond Hindenes (@trondhindenes)
- Jordan Borean (@jborean93)
'''

EXAMPLES = r'''
- name: Install the Visual C thingy
  ansible.windows.win_package:
    path: http://download.microsoft.com/download/1/6/B/16B06F60-3B20-4FF2-B699-5E9B7962F9AE/VSU_4/vcredist_x64.exe
    product_id: '{CF2BEA3C-26EA-32F8-AA9B-331F7E34BA97}'
    arguments: /install /passive /norestart

- name: Install Visual C thingy with list of arguments instead of a string
  ansible.windows.win_package:
    path: http://download.microsoft.com/download/1/6/B/16B06F60-3B20-4FF2-B699-5E9B7962F9AE/VSU_4/vcredist_x64.exe
    product_id: '{CF2BEA3C-26EA-32F8-AA9B-331F7E34BA97}'
    arguments:
    - /install
    - /passive
    - /norestart

- name: Install MSBuild thingy with arguments split to prevent quotes
  ansible.windows.win_package:
    path: https://download.visualstudio.microsoft.com/download/pr/9665567e-f580-4acd-85f2-bc94a1db745f/vs_BuildTools.exe
    product_id: '{D1437F51-786A-4F57-A99C-F8E94FBA1BD8}'
    arguments:
    - --norestart
    - --passive
    - --wait
    - --add
    - Microsoft.Net.Component.4.6.1.TargetingPack
    - --add
    - Microsoft.Net.Component.4.6.TargetingPack

- name: Install Remote Desktop Connection Manager from msi with a permanent log
  ansible.windows.win_package:
    path: https://download.microsoft.com/download/A/F/0/AF0071F3-B198-4A35-AA90-C68D103BDCCF/rdcman.msi
    product_id: '{0240359E-6A4C-4884-9E94-B397A02D893C}'
    state: present
    log_path: D:\logs\vcredist_x64-exe-{{lookup('pipe', 'date +%Y%m%dT%H%M%S')}}.log

- name: Install Application from msi with multiple properties for installer
  ansible.windows.win_package:
    path: C:\temp\Application.msi
    state: present
    arguments: >-
      SERVICE=1
      DBNAME=ApplicationDB
      DBSERVER=.\SQLEXPRESS
      INSTALLDIR="C:\Program Files (x86)\App lication\App Server"

- name: Install Microsoft® SQL Server® 2019 Express (DPAPI example)
  ansible.windows.win_package:
    path: C:\temp\SQLEXPR_x64_ENU\SETUP.EXE
    product_id: Microsoft SQL Server SQL2019
    arguments:
      - SAPWD=VeryHardPassword
      - /ConfigurationFile=C:\temp\configuration.ini
  become: true
  vars:
    ansible_become_method: runas
    ansible_become_user: "{{ user }}"
    ansible_become_pass: "{{ password }}"

- name: Uninstall Remote Desktop Connection Manager
  ansible.windows.win_package:
    product_id: '{0240359E-6A4C-4884-9E94-B397A02D893C}'
    state: absent

- name: Install Remote Desktop Connection Manager locally omitting the product_id
  ansible.windows.win_package:
    path: C:\temp\rdcman.msi
    state: present

- name: Uninstall Remote Desktop Connection Manager from local MSI omitting the product_id
  ansible.windows.win_package:
    path: C:\temp\rdcman.msi
    state: absent

# 7-Zip exe doesn't use a guid for the Product ID
- name: Install 7zip from a network share with specific credentials
  ansible.windows.win_package:
    path: \\domain\programs\7z.exe
    product_id: 7-Zip
    arguments: /S
    state: present
  become: true
  become_method: runas
  become_flags: logon_type=new_credential logon_flags=netcredentials_only
  vars:
    ansible_become_user: DOMAIN\User
    ansible_become_password: Password

- name: Install 7zip and use a file version for the installation check
  ansible.windows.win_package:
    path: C:\temp\7z.exe
    creates_path: C:\Program Files\7-Zip\7z.exe
    creates_version: 16.04
    state: present

- name: Uninstall 7zip from the exe
  ansible.windows.win_package:
    path: C:\Program Files\7-Zip\Uninstall.exe
    product_id: 7-Zip
    arguments: /S
    state: absent

- name: Uninstall 7zip without specifying the path
  ansible.windows.win_package:
    product_id: 7-Zip
    arguments: /S
    state: absent

- name: Install application and override expected return codes
  ansible.windows.win_package:
    path: https://download.microsoft.com/download/1/6/7/167F0D79-9317-48AE-AEDB-17120579F8E2/NDP451-KB2858728-x86-x64-AllOS-ENU.exe
    product_id: '{7DEBE4EB-6B40-3766-BB35-5CBBC385DA37}'
    arguments: '/q /norestart'
    state: present
    expected_return_code: [0, 666, 3010]

- name: Install a .msp patch
  ansible.windows.win_package:
    path: C:\Patches\Product.msp
    state: present

- name: Remove a .msp patch
  ansible.windows.win_package:
    product_id: '{AC76BA86-A440-FFFF-A440-0C13154E5D00}'
    state: absent

- name: Enable installation of 3rd party MSIX packages
  ansible.windows.win_regedit:
    path: HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\AppModelUnlock
    name: AllowAllTrustedApps
    data: 1
    type: dword
    state: present

- name: Install an MSIX package for the current user
  ansible.windows.win_package:
    path: C:\Installers\Calculator.msix  # Can be .appx, .msixbundle, or .appxbundle
    state: present

- name: Uninstall an MSIX package using the product_id
  ansible.windows.win_package:
    product_id: InputApp
    state: absent
'''

RETURN = r'''
log:
  description: The contents of the MSI or MSP log.
  returned: installation/uninstallation failure for MSI or MSP packages
  type: str
  sample: Installation completed successfully
rc:
  description: The return code of the package process.
  returned: change occurred
  type: int
  sample: 0
reboot_required:
  description: Whether a reboot is required to finalise package. This is set
    to true if the executable return code is 3010.
  returned: always
  type: bool
  sample: true
stdout:
  description: The stdout stream of the package process.
  returned: failure during install or uninstall
  type: str
  sample: Installing program
stderr:
  description: The stderr stream of the package process.
  returned: failure during install or uninstall
  type: str
  sample: Failed to install program
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: cloudformation_exports_info
short_description: Read a value from CloudFormation Exports
version_added: 1.0.0
description:
  - Module retrieves a value from CloudFormation Exports
author:
  - "Michael Moyle (@mmoyle)"
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.
- name: Get Exports
  community.aws.cloudformation_exports_info: {}
  register: cf_exports
- ansible.builtin.debug:
    msg: "{{ cf_exports }}"
"""

RETURN = r"""
export_items:
    description: A dictionary of Exports items names and values.
    returned: Always
    type: dict
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


@AWSRetry.exponential_backoff()
def list_exports(cloudformation_client):
    """Get Exports Names and Values and return in dictionary"""
    list_exports_paginator = cloudformation_client.get_paginator("list_exports")
    exports = list_exports_paginator.paginate().build_full_result()["Exports"]
    export_items = dict()

    for item in exports:
        export_items[item["Name"]] = item["Value"]

    return export_items


def main():
    argument_spec = dict()
    result = dict(changed=False, original_message="")

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)
    cloudformation_client = module.client("cloudformation")

    try:
        result["export_items"] = list_exports(cloudformation_client)

    except (ClientError, BotoCoreError) as e:
        module.fail_json_aws(e)

    result.update()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Cole Neubauer, (@coleneubauer)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_backuppolicy_info
version_added: "1.4.0"
short_description: Get Info on Azure Backup Policy
description:
    - Create and delete instance of Azure Backup Policy.

options:
    vault_name:
        description:
            - The name of the Recovery Services Vault the policy belongs to.
        required: true
        type: str
    name:
        description:
            - The name of the backup policy.
        required: true
        type: str
    resource_group:
        description:
            - The name of the resource group the vault is in.
        required: true
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Cole Neubauer(@coleneubauer)
'''

EXAMPLES = '''
- name: Get backup policy information
  azure_rm_backuppolicy_info:
    vault_name: Vault_Name
    name: Policy_Name
    resource_group: Resource_Group_Name
  register: backup_policy
'''

RETURN = '''
id:
    description:
        - Id of specified backup policy.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.RecoveryServices/vaults/Vault_Name/backupPolicies/Policy_Name"
location:
    description:
        - Location of backup policy.
    type: str
    returned: always
    sample: eastus
name:
    description:
        - Name of backup policy.
    type: str
    returned: always
    sample: DefaultPolicy
type:
    description:
        - Type of backup policy.
    type: str
    returned: always
    sample: Microsoft.RecoveryServices/vaults/backupPolicies
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError

except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMBackupPolicyInfo(AzureRMModuleBase):
    """Information class for an Azure RM Backup Policy"""

    def __init__(self):
        self.module_arg_spec = dict(
            vault_name=dict(type='str', required=True),
            name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
        )

        self.vault_name = None
        self.name = None
        self.resource_group = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            id=None,
            changed=False
        )

        super(AzureRMBackupPolicyInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                      supports_check_mode=True,
                                                      supports_tags=False,
                                                      facts_module=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        existing_backup_policy = None
        response = None

        existing_backup_policy = self.get_backup_policy()

        self.set_results(existing_backup_policy)

        return self.results

    def get_backup_policy(self):
        '''
        Gets the properties of the specified backup policy.

        :return: ProtectionPolicyResource
        '''
        self.log("Checking if the backup policy {0} for vault {1} in resource group {2} is present".format(self.name,
                                                                                                           self.vault_name,
                                                                                                           self.resource_group))

        policy = None

        try:
            policy = self.recovery_services_backup_client.protection_policies.get(vault_name=self.vault_name,
                                                                                  resource_group_name=self.resource_group,
                                                                                  policy_name=self.name)
        except ResourceNotFoundError as ex:
            self.log("Could not find backup policy {0} for vault {1} in resource group {2}".format(self.name, self.vault_name, self.resource_group))

        return policy

    def set_results(self, policy):
        if policy:
            self.results['id'] = policy.id
            self.results['location'] = policy.location
            self.results['name'] = policy.name
            self.results['type'] = policy.type

        else:
            self.results['id'] = None
            self.results['location'] = None
            self.results['name'] = None
            self.results['type'] = None


def main():
    """Main execution"""
    AzureRMBackupPolicyInfo()


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr acl_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.acl_interfaces.acl_interfaces import (
    Acl_interfacesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.acl_interfaces import (
    Acl_interfacesTemplate,
)


class Acl_interfacesFacts(object):
    """The iosxr acl_interfaces fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Acl_interfacesArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_config(self, connection):
        return connection.get_config(flags="interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for acl_interfaces
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """

        if not data:
            data = self.get_config(connection)

        config_parser = Acl_interfacesTemplate(lines=data.splitlines())
        entry = sorted(
            list(config_parser.parse().values()),
            key=lambda k, sk="name": k[sk],
        )
        if entry:
            for item in entry:
                item["access_groups"] = sorted(
                    list(item["access_groups"].values()),
                    key=lambda k, sk="afi": k[sk],
                )

        ansible_facts["ansible_network_resources"].pop("acl_interfaces", None)
        facts = {"acl_interfaces": []}
        params = utils.validate_config(self.argument_spec, {"config": entry})
        for cfg in params["config"]:
            facts["acl_interfaces"].append(utils.remove_empties(cfg))

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_subnet
version_added: 1.0.0
short_description: Manage subnets in AWS virtual private clouds
description:
  - Manage subnets in AWS virtual private clouds.
author:
  - Robert Estelle (@erydo)
  - Brad Davidson (@brandond)
options:
  az:
    description:
      - The availability zone for the subnet.
      - Required if I(outpost_arn) is set.
    type: str
  cidr:
    description:
      - The CIDR block for the subnet. E.g. C(192.0.2.0/24).
    type: str
    required: true
  ipv6_cidr:
    description:
      - The IPv6 CIDR block for the subnet.
      - The VPC must have a /56 block assigned and this value must be a valid IPv6 /64 that falls in the VPC range.
      - Required if I(assign_instances_ipv6=true)
    type: str
    default: ''
  outpost_arn:
    description:
      - The Amazon Resource Name (ARN) of the Outpost.
      - If set, allows to create subnet in an Outpost.
      - If I(outpost_arn) is set, I(az) must also be specified.
    type: str
    default: ''
  state:
    description:
      - Create or remove the subnet.
    default: present
    choices: [ 'present', 'absent' ]
    type: str
  vpc_id:
    description:
      -"VPC ID of the VPC in which to create or delete the subnet.
    required: true
    type: str
  map_public:
    description:
      - Whether instances launched into the subnet should default to being assigned public IP address.
    type: bool
    default: false
  assign_instances_ipv6:
    description:
      - Whether instances launched into the subnet should default to being automatically assigned an IPv6 address.
      - If I(assign_instances_ipv6=true), I(ipv6_cidr) must also be specified.
    type: bool
    default: false
  wait:
    description:
      - Whether to wait for changes to complete.
    type: bool
    default: true
  wait_timeout:
    description:
      - Number of seconds to wait for changes to complete
      - Ignored unless I(wait=True).
    default: 300
    type: int
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Create subnet for database servers
  amazon.aws.ec2_vpc_subnet:
    state: present
    vpc_id: vpc-123456
    cidr: 10.0.1.16/28
    tags:
      Name: Database Subnet
  register: database_subnet

- name: Remove subnet for database servers
  amazon.aws.ec2_vpc_subnet:
    state: absent
    vpc_id: vpc-123456
    cidr: 10.0.1.16/28

- name: Create subnet with IPv6 block assigned
  amazon.aws.ec2_vpc_subnet:
    state: present
    vpc_id: vpc-123456
    cidr: 10.1.100.0/24
    ipv6_cidr: 2001:db8:0:102::/64

- name: Remove IPv6 block assigned to subnet
  amazon.aws.ec2_vpc_subnet:
    state: present
    vpc_id: vpc-123456
    cidr: 10.1.100.0/24
    ipv6_cidr: ''
"""

RETURN = r"""
subnet:
    description: Dictionary of subnet values
    returned: I(state=present)
    type: complex
    contains:
        id:
            description: Subnet resource id
            returned: I(state=present)
            type: str
            sample: subnet-b883b2c4
        cidr_block:
            description: The IPv4 CIDR of the Subnet
            returned: I(state=present)
            type: str
            sample: "10.0.0.0/16"
        ipv6_cidr_block:
            description: The IPv6 CIDR block actively associated with the Subnet
            returned: I(state=present)
            type: str
            sample: "2001:db8:0:102::/64"
        availability_zone:
            description: Availability zone of the Subnet
            returned: I(state=present)
            type: str
            sample: us-east-1a
        state:
            description: state of the Subnet
            returned: I(state=present)
            type: str
            sample: available
        tags:
            description: tags attached to the Subnet, includes name
            returned: I(state=present)
            type: dict
            sample: {"Name": "My Subnet", "env": "staging"}
        map_public_ip_on_launch:
            description: whether public IP is auto-assigned to new instances
            returned: I(state=present)
            type: bool
            sample: false
        assign_ipv6_address_on_creation:
            description: whether IPv6 address is auto-assigned to new instances
            returned: I(state=present)
            type: bool
            sample: false
        vpc_id:
            description: the id of the VPC where this Subnet exists
            returned: I(state=present)
            type: str
            sample: vpc-67236184
        available_ip_address_count:
            description: number of available IPv4 addresses
            returned: I(state=present)
            type: str
            sample: 251
        default_for_az:
            description: indicates whether this is the default Subnet for this Availability Zone
            returned: I(state=present)
            type: bool
            sample: false
        ipv6_association_id:
            description: The IPv6 association ID for the currently associated CIDR
            returned: I(state=present)
            type: str
            sample: subnet-cidr-assoc-b85c74d2
        ipv6_cidr_block_association_set:
            description: An array of IPv6 cidr block association set information.
            returned: I(state=present)
            type: complex
            contains:
                association_id:
                    description: The association ID
                    returned: always
                    type: str
                ipv6_cidr_block:
                    description: The IPv6 CIDR block that is associated with the subnet.
                    returned: always
                    type: str
                ipv6_cidr_block_state:
                    description: A hash/dict that contains a single item. The state of the cidr block association.
                    returned: always
                    type: dict
                    contains:
                        state:
                            description: The CIDR block association state.
                            returned: always
                            type: str
"""


import time

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.arn import is_outpost_arn
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_tag_filter_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_specifications
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list
from ansible_collections.amazon.aws.plugins.module_utils.waiters import get_waiter


def get_subnet_info(subnet):
    if "Subnets" in subnet:
        return [get_subnet_info(s) for s in subnet["Subnets"]]
    elif "Subnet" in subnet:
        subnet = camel_dict_to_snake_dict(subnet["Subnet"])
    else:
        subnet = camel_dict_to_snake_dict(subnet)

    if "tags" in subnet:
        subnet["tags"] = boto3_tag_list_to_ansible_dict(subnet["tags"])
    else:
        subnet["tags"] = dict()

    if "subnet_id" in subnet:
        subnet["id"] = subnet["subnet_id"]
        del subnet["subnet_id"]

    subnet["ipv6_cidr_block"] = ""
    subnet["ipv6_association_id"] = ""
    ipv6set = subnet.get("ipv6_cidr_block_association_set")
    if ipv6set:
        for item in ipv6set:
            if item.get("ipv6_cidr_block_state", {}).get("state") in ("associated", "associating"):
                subnet["ipv6_cidr_block"] = item["ipv6_cidr_block"]
                subnet["ipv6_association_id"] = item["association_id"]

    return subnet


def waiter_params(module, params, start_time):
    remaining_wait_timeout = int(module.params["wait_timeout"] + start_time - time.time())
    params["WaiterConfig"] = {"Delay": 5, "MaxAttempts": remaining_wait_timeout // 5}
    return params


def handle_waiter(conn, module, waiter_name, params, start_time):
    try:
        get_waiter(conn, waiter_name).wait(**waiter_params(module, params, start_time))
    except botocore.exceptions.WaiterError as e:
        module.fail_json_aws(e, "Failed to wait for updates to complete")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, "An exception happened while trying to wait for updates")


def create_subnet(conn, module, vpc_id, cidr, tags, ipv6_cidr=None, outpost_arn=None, az=None, start_time=None):
    wait = module.params["wait"]

    params = dict(VpcId=vpc_id, CidrBlock=cidr)

    if ipv6_cidr:
        params["Ipv6CidrBlock"] = ipv6_cidr

    if az:
        params["AvailabilityZone"] = az

    if tags:
        params["TagSpecifications"] = boto3_tag_specifications(tags, types="subnet")

    if outpost_arn:
        if is_outpost_arn(outpost_arn):
            params["OutpostArn"] = outpost_arn
        else:
            module.fail_json("OutpostArn does not match the pattern specified in API specifications.")

    try:
        subnet = get_subnet_info(conn.create_subnet(aws_retry=True, **params))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't create subnet")

    # Sometimes AWS takes its time to create a subnet and so using
    # new subnets's id to do things like create tags results in
    # exception.
    if wait and subnet.get("state") != "available":
        handle_waiter(conn, module, "subnet_exists", {"SubnetIds": [subnet["id"]]}, start_time)
        handle_waiter(conn, module, "subnet_available", {"SubnetIds": [subnet["id"]]}, start_time)
        subnet["state"] = "available"

    return subnet


def ensure_tags(conn, module, subnet, tags, purge_tags, start_time):
    changed = ensure_ec2_tags(
        conn,
        module,
        subnet["id"],
        resource_type="subnet",
        purge_tags=purge_tags,
        tags=tags,
        retry_codes=["InvalidSubnetID.NotFound"],
    )

    if not changed:
        return changed

    if module.params["wait"] and not module.check_mode:
        # Wait for tags to be updated
        filters = ansible_dict_to_boto3_filter_list(ansible_dict_to_tag_filter_dict(tags))
        handle_waiter(conn, module, "subnet_exists", {"SubnetIds": [subnet["id"]], "Filters": filters}, start_time)

    return changed


def ensure_map_public(conn, module, subnet, map_public, check_mode, start_time):
    if check_mode:
        return
    try:
        conn.modify_subnet_attribute(aws_retry=True, SubnetId=subnet["id"], MapPublicIpOnLaunch={"Value": map_public})
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't modify subnet attribute")


def ensure_assign_ipv6_on_create(conn, module, subnet, assign_instances_ipv6, check_mode, start_time):
    if check_mode:
        return
    try:
        conn.modify_subnet_attribute(
            aws_retry=True, SubnetId=subnet["id"], AssignIpv6AddressOnCreation={"Value": assign_instances_ipv6}
        )
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't modify subnet attribute")


def disassociate_ipv6_cidr(conn, module, subnet, start_time):
    if subnet.get("assign_ipv6_address_on_creation"):
        ensure_assign_ipv6_on_create(conn, module, subnet, False, False, start_time)

    try:
        conn.disassociate_subnet_cidr_block(aws_retry=True, AssociationId=subnet["ipv6_association_id"])
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(
            e,
            msg=f"Couldn't disassociate ipv6 cidr block id {subnet['ipv6_association_id']} from subnet {subnet['id']}",
        )

    # Wait for cidr block to be disassociated
    if module.params["wait"]:
        filters = ansible_dict_to_boto3_filter_list(
            {"ipv6-cidr-block-association.state": ["disassociated"], "vpc-id": subnet["vpc_id"]}
        )
        handle_waiter(conn, module, "subnet_exists", {"SubnetIds": [subnet["id"]], "Filters": filters}, start_time)


def ensure_ipv6_cidr_block(conn, module, subnet, ipv6_cidr, check_mode, start_time):
    wait = module.params["wait"]
    changed = False

    if subnet["ipv6_association_id"] and not ipv6_cidr:
        if not check_mode:
            disassociate_ipv6_cidr(conn, module, subnet, start_time)
        changed = True

    if ipv6_cidr:
        filters = ansible_dict_to_boto3_filter_list(
            {"ipv6-cidr-block-association.ipv6-cidr-block": ipv6_cidr, "vpc-id": subnet["vpc_id"]}
        )

        try:
            _subnets = conn.describe_subnets(aws_retry=True, Filters=filters)
            check_subnets = get_subnet_info(_subnets)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't get subnet info")

        if check_subnets and check_subnets[0]["ipv6_cidr_block"]:
            module.fail_json(msg=f"The IPv6 CIDR '{ipv6_cidr}' conflicts with another subnet")

        if subnet["ipv6_association_id"]:
            if not check_mode:
                disassociate_ipv6_cidr(conn, module, subnet, start_time)
            changed = True

        try:
            if not check_mode:
                associate_resp = conn.associate_subnet_cidr_block(
                    aws_retry=True, SubnetId=subnet["id"], Ipv6CidrBlock=ipv6_cidr
                )
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg=f"Couldn't associate ipv6 cidr {ipv6_cidr} to {subnet['id']}")
        else:
            if not check_mode and wait:
                filters = ansible_dict_to_boto3_filter_list(
                    {"ipv6-cidr-block-association.state": ["associated"], "vpc-id": subnet["vpc_id"]}
                )
                handle_waiter(
                    conn, module, "subnet_exists", {"SubnetIds": [subnet["id"]], "Filters": filters}, start_time
                )

        if associate_resp.get("Ipv6CidrBlockAssociation", {}).get("AssociationId"):
            subnet["ipv6_association_id"] = associate_resp["Ipv6CidrBlockAssociation"]["AssociationId"]
            subnet["ipv6_cidr_block"] = associate_resp["Ipv6CidrBlockAssociation"]["Ipv6CidrBlock"]
            if subnet["ipv6_cidr_block_association_set"]:
                subnet["ipv6_cidr_block_association_set"][0] = camel_dict_to_snake_dict(
                    associate_resp["Ipv6CidrBlockAssociation"]
                )
            else:
                subnet["ipv6_cidr_block_association_set"].append(
                    camel_dict_to_snake_dict(associate_resp["Ipv6CidrBlockAssociation"])
                )

    return changed


def _matching_subnet_filters(vpc_id, cidr):
    return ansible_dict_to_boto3_filter_list({"vpc-id": vpc_id, "cidr-block": cidr})


def get_matching_subnet(conn, module, vpc_id, cidr):
    filters = _matching_subnet_filters(vpc_id, cidr)
    try:
        _subnets = conn.describe_subnets(aws_retry=True, Filters=filters)
        subnets = get_subnet_info(_subnets)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't get matching subnet")

    if subnets:
        return subnets[0]

    return None


def ensure_subnet_present(conn, module):
    subnet = get_matching_subnet(conn, module, module.params["vpc_id"], module.params["cidr"])
    changed = False

    # Initialize start so max time does not exceed the specified wait_timeout for multiple operations
    start_time = time.time()

    if subnet is None:
        if not module.check_mode:
            subnet = create_subnet(
                conn,
                module,
                module.params["vpc_id"],
                module.params["cidr"],
                module.params["tags"],
                ipv6_cidr=module.params["ipv6_cidr"],
                outpost_arn=module.params["outpost_arn"],
                az=module.params["az"],
                start_time=start_time,
            )
        changed = True
        # Subnet will be None when check_mode is true
        if subnet is None:
            return {"changed": changed, "subnet": {}}
    if module.params["wait"]:
        handle_waiter(conn, module, "subnet_exists", {"SubnetIds": [subnet["id"]]}, start_time)

    if module.params["ipv6_cidr"] != subnet.get("ipv6_cidr_block"):
        if ensure_ipv6_cidr_block(conn, module, subnet, module.params["ipv6_cidr"], module.check_mode, start_time):
            changed = True

    if module.params["map_public"] != subnet["map_public_ip_on_launch"]:
        ensure_map_public(conn, module, subnet, module.params["map_public"], module.check_mode, start_time)
        changed = True

    if module.params["assign_instances_ipv6"] != subnet.get("assign_ipv6_address_on_creation"):
        ensure_assign_ipv6_on_create(
            conn, module, subnet, module.params["assign_instances_ipv6"], module.check_mode, start_time
        )
        changed = True

    if ensure_tags(conn, module, subnet, module.params["tags"], module.params["purge_tags"], start_time):
        changed = True

    subnet = get_matching_subnet(conn, module, module.params["vpc_id"], module.params["cidr"])
    if not module.check_mode and module.params["wait"]:
        subnet_filter = _matching_subnet_filters(module.params["vpc_id"], module.params["cidr"])
        handle_waiter(conn, module, "subnet_exists", {"Filters": subnet_filter}, start_time)
        subnet = get_matching_subnet(conn, module, module.params["vpc_id"], module.params["cidr"])
        if not subnet:
            module.fail_json("Failed to describe newly created subnet")
        # GET calls are not monotonic for map_public_ip_on_launch and assign_ipv6_address_on_creation
        # so we only wait for those if necessary just before returning the subnet
        subnet = ensure_final_subnet(conn, module, subnet, start_time)

    return {"changed": changed, "subnet": subnet}


def ensure_final_subnet(conn, module, subnet, start_time):
    for _rewait in range(0, 30):
        map_public_correct = False
        assign_ipv6_correct = False

        if module.params["map_public"] == subnet["map_public_ip_on_launch"]:
            map_public_correct = True
        else:
            if module.params["map_public"]:
                handle_waiter(conn, module, "subnet_has_map_public", {"SubnetIds": [subnet["id"]]}, start_time)
            else:
                handle_waiter(conn, module, "subnet_no_map_public", {"SubnetIds": [subnet["id"]]}, start_time)

        if module.params["assign_instances_ipv6"] == subnet.get("assign_ipv6_address_on_creation"):
            assign_ipv6_correct = True
        else:
            if module.params["assign_instances_ipv6"]:
                handle_waiter(conn, module, "subnet_has_assign_ipv6", {"SubnetIds": [subnet["id"]]}, start_time)
            else:
                handle_waiter(conn, module, "subnet_no_assign_ipv6", {"SubnetIds": [subnet["id"]]}, start_time)

        if map_public_correct and assign_ipv6_correct:
            break

        time.sleep(5)
        subnet = get_matching_subnet(conn, module, module.params["vpc_id"], module.params["cidr"])

    return subnet


def ensure_subnet_absent(conn, module):
    subnet = get_matching_subnet(conn, module, module.params["vpc_id"], module.params["cidr"])
    if subnet is None:
        return {"changed": False}

    try:
        if not module.check_mode:
            conn.delete_subnet(aws_retry=True, SubnetId=subnet["id"])
            if module.params["wait"]:
                handle_waiter(conn, module, "subnet_deleted", {"SubnetIds": [subnet["id"]]}, time.time())
        return {"changed": True}
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't delete subnet")


def main():
    argument_spec = dict(
        az=dict(default=None, required=False),
        cidr=dict(required=True),
        ipv6_cidr=dict(default="", required=False),
        outpost_arn=dict(default="", type="str", required=False),
        state=dict(default="present", choices=["present", "absent"]),
        tags=dict(required=False, type="dict", aliases=["resource_tags"]),
        vpc_id=dict(required=True),
        map_public=dict(default=False, required=False, type="bool"),
        assign_instances_ipv6=dict(default=False, required=False, type="bool"),
        wait=dict(type="bool", default=True),
        wait_timeout=dict(type="int", default=300, required=False),
        purge_tags=dict(default=True, type="bool"),
    )

    required_if = [("assign_instances_ipv6", True, ["ipv6_cidr"])]

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True, required_if=required_if)

    if module.params.get("outpost_arn") and not module.params.get("az"):
        module.fail_json(msg="To specify OutpostArn, you must specify the Availability Zone of the Outpost subnet.")

    if module.params.get("assign_instances_ipv6") and not module.params.get("ipv6_cidr"):
        module.fail_json(msg="assign_instances_ipv6 is True but ipv6_cidr is None or an empty string")

    retry_decorator = AWSRetry.jittered_backoff(retries=10)
    connection = module.client("ec2", retry_decorator=retry_decorator)

    state = module.params.get("state")

    try:
        if state == "present":
            result = ensure_subnet_present(connection, module)
        elif state == "absent":
            result = ensure_subnet_absent(connection, module)
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)

    module.exit_json(**result)


if __name__ == "__main__":
    main()
